@@ -962,9 +962,7 @@ impl TbsCert {
                     subject_public_key_info.to_subject_name()
                 )
             },
-            subject_key_identifier: {
-                KeyIdentifier::from_public_key(&subject_public_key_info)
-            },
+            subject_key_identifier: subject_public_key_info.key_identifier(),
             subject_public_key_info,
             basic_ca: None,
             authority_key_identifier: None,
@@ -1053,7 +1051,7 @@ impl TbsCert {
     /// the public key itself as well as the `subject_public_key_identifier`
     /// to the identifier of that key.
     pub fn set_subject_public_key(&mut self, key: PublicKey) {
-        self.subject_key_identifier = KeyIdentifier::from_public_key(&key);
+        self.subject_key_identifier = key.key_identifier();
         self.subject_public_key_info = key;
     }
 
@@ -2699,7 +2697,7 @@ mod signer_test {
     use crate::repository::cert::Cert;
     use crate::repository::crypto::PublicKeyFormat;
     use crate::repository::crypto::softsigner::OpenSslSigner;
-    use crate::repository::resources::{AsId, Prefix};
+    use crate::repository::resources::{Asn, Prefix};
     use crate::repository::tal::TalInfo;
     use super::*;
 
@@ -2720,7 +2718,7 @@ mod signer_test {
         cert.set_rpki_manifest(Some(uri));
         cert.build_v4_resource_blocks(|b| b.push(Prefix::new(0, 0)));
         cert.build_v6_resource_blocks(|b| b.push(Prefix::new(0, 0)));
-        cert.build_as_resource_blocks(|b| b.push((AsId::MIN, AsId::MAX)));
+        cert.build_as_resource_blocks(|b| b.push((Asn::MIN, Asn::MAX)));
         let cert = cert.into_cert(&signer, &key).unwrap().to_captured();
         let cert = Cert::decode(cert.as_slice()).unwrap();
         let talinfo = TalInfo::from_name("foo".into()).into_arc();
