@@ -743,7 +743,7 @@ mod signer_test {
 
     #[test]
     fn build_ta_cert() {
-        let mut signer = OpenSslSigner::new();
+        let signer = OpenSslSigner::new();
         let key = signer.create_key(PublicKeyFormat::Rsa).unwrap();
         let pubkey = signer.get_key_info(&key).unwrap();
         let crl = TbsCertList::new(
@@ -752,7 +752,7 @@ mod signer_test {
             Time::now(),
             Time::tomorrow(),
             vec![CrlEntry::new(12u64.into(), Time::now())],
-            KeyIdentifier::from_public_key(&pubkey),
+            pubkey.key_identifier(),
             12u64.into()
         );
         let crl = crl.into_crl(&signer, &key).unwrap().to_captured();
