@@ -377,7 +377,7 @@ impl Iterator for FileListIter {
 /// An entry in the manifest file list.
 ///
 /// This type contains a file name and a hash over the file. Both are
-/// expressed through generic types for superiour flexibility.
+/// expressed through generic types for superior flexibility.
 #[derive(Clone, Debug)]
 pub struct FileAndHash<F, H> {
     /// The name of a file.
@@ -539,13 +539,13 @@ mod signer_test {
     use crate::repository::cert::{KeyUsage, Overclaim, TbsCert};
     use crate::repository::crypto::{PublicKeyFormat, Signer};
     use crate::repository::crypto::softsigner::OpenSslSigner;
-    use crate::repository::resources::{AsId, Prefix};
+    use crate::repository::resources::{Asn, Prefix};
     use crate::repository::tal::TalInfo;
     use crate::repository::x509::Validity;
     use super::*;
 
     fn make_test_manifest() -> Manifest {
-        let mut signer = OpenSslSigner::new();
+        let signer = OpenSslSigner::new();
         let key = signer.create_key(PublicKeyFormat::Rsa).unwrap();
         let pubkey = signer.get_key_info(&key).unwrap();
         let uri = uri::Rsync::from_str("rsync://example.com/m/p").unwrap();
@@ -560,7 +560,7 @@ mod signer_test {
         cert.set_rpki_manifest(Some(uri.clone()));
         cert.build_v4_resource_blocks(|b| b.push(Prefix::new(0, 0)));
         cert.build_v6_resource_blocks(|b| b.push(Prefix::new(0, 0)));
-        cert.build_as_resource_blocks(|b| b.push((AsId::MIN, AsId::MAX)));
+        cert.build_as_resource_blocks(|b| b.push((Asn::MIN, Asn::MAX)));
         let cert = cert.into_cert(&signer, &key).unwrap();
 
         let content = ManifestContent::new(
@@ -575,7 +575,7 @@ mod signer_test {
         let manifest = content.into_manifest(
             SignedObjectBuilder::new(
                 12u64.into(), Validity::from_secs(86400), uri.clone(),
-                uri.clone(), uri.clone()
+                uri.clone(), uri
             ),
             &signer, &key
         ).unwrap();
