@@ -1037,34 +1037,19 @@ impl AddressRange {
         family: AddressFamily,
     ) -> Result<Self, S::Err> {
         let cons = content.as_constructed()?;
-        let min = Self::check_len(Prefix::take_from(cons)?, family)?;
-        let max = Self::check_len(Prefix::take_from(cons)?, family)?;
+        let min = Prefix::take_from(cons)?;
+        let max = Prefix::take_from(cons)?;
+        if min.addr_len() > family.max_addr_len()
+            || max.addr_len() > family.max_addr_len()
+        {
+            return Err(decode::Malformed.into())
+        }
         Ok(AddressRange {
             min: min.min(),
             max: max.max(),
         })
     }
 
-    #[cfg(not(feature = "compat"))]
-    fn check_len(
-        addr: Prefix, family: AddressFamily
-    ) -> Result<Prefix, decode::Error> {
-        if addr.addr_len() > family.max_addr_len() {
-            Err(decode::Malformed)
-        }
-        else {
-            Ok(addr)
-        }
-    }
-
-    #[cfg(feature = "compat")]
-    fn check_len(
-        mut addr: Prefix, family: AddressFamily
-    ) -> Result<Prefix, decode::Error> {
-        addr.len = std::cmp::min(addr.len, family.max_addr_len());
-        Ok(addr)
-    }
-
     /*
     /// Skips over the address range at the beginning of a value.
     fn skip_opt_in<S: decode::Source>(
@@ -1445,7 +1430,6 @@ impl Addr {
     }
 
     /// Returns a byte array for the address.
-    #[allow(clippy::transmute_num_to_bytes)]
     pub fn to_bytes(self) -> [u8; 16] {
         unsafe { mem::transmute(self.0.to_be()) }
     }
