@@ -1,7 +1,7 @@
 //! Resource Tagged Attestations.
 //!
-//! Resouce Tagged Attestations attaching signed attestations of ownership of 
-//! resources referred to by a docuement. This is currently an IETF draft,
+//! Resource Tagged Attestations attaching signed attestations of ownership of 
+//! resources referred to by a document. This is currently an IETF draft,
 //! see [draft-michaelson-rpki-rta] for details.
 //!
 //! Current limitations:
@@ -9,7 +9,7 @@
 //! * Resources are read in a rather relaxed way: they can be unordered and
 //!   overlapping. This happens even if the strict flag is set.
 //!
-//! draft-michaelson-rpki-rta: https://tools.ietf.org/html/draft-michaelson-rpki-rta
+//! [draft-michaelson-rpki-rta]: https://tools.ietf.org/html/draft-michaelson-rpki-rta
 
 use std::ops;
 use bcder::{decode, encode};
@@ -183,13 +183,17 @@ impl ResourceTaggedAttestation {
                                 if v4.is_some() {
                                     xerr!(return Err(decode::Malformed.into()));
                                 }
-                                v4 = Some(IpBlocks::take_from(cons)?);
+                                v4 = Some(IpBlocks::take_from_with_family(
+                                    cons, AddressFamily::Ipv4
+                                )?);
                             }
                             AddressFamily::Ipv6 => {
                                 if v6.is_some() {
                                     xerr!(return Err(decode::Malformed.into()));
                                 }
-                                v6 = Some(IpBlocks::take_from(cons)?);
+                                v6 = Some(IpBlocks::take_from_with_family(
+                                    cons, AddressFamily::Ipv6
+                                )?);
                             }
                         }
                         Ok(())
@@ -419,6 +423,16 @@ pub struct SignerInfo {
 }
 
 impl SignerInfo {
+    /// Returns the signing time if available.
+    pub fn signing_time(&self) -> Option<Time> {
+        self.signing_time
+    }
+
+    /// Returns the binary signing time if available.
+    pub fn binary_signing_time(&self) -> Option<u64> {
+        self.binary_signing_time
+    }
+
     pub fn take_opt_from<S: decode::Source>(
         cons: &mut decode::Constructed<S>,
     ) -> Result<Option<Self>, S::Err> {
@@ -672,7 +686,7 @@ impl<'a> Ca<'a> {
 
 #[derive(Clone, Debug)]
 struct Chain<'a> {
-    /// The topmost certifcate of the path.
+    /// The topmost certificate of the path.
     cert: &'a Cert,
 
     /// The resources of the topmost cert.
@@ -896,7 +910,7 @@ impl<'a> Chain<'a> {
         // Now check properly if `ca` should be our CA and return if it isn’t.
         //
         // If it isn’t not all is lost. There may just be a key identifier
-        // collission or something. Unlikely but hey.
+        // collision or something. Unlikely but hey.
         if self.cert.verify_issuer_claim(ca, strict).is_err()
             || self.cert.verify_signature(ca, strict).is_err()
         {
@@ -1263,7 +1277,7 @@ impl RtaBuilder {
         )?;
 
         self.signer_infos.push(SignerInfo {
-            sid: KeyIdentifier::from_public_key(&signer.get_key_info(key)?),
+            sid: signer.get_key_info(key)?.key_identifier(),
             digest_algorithm: self.digest_algorithm,
             signed_attrs,
             signature,
