@@ -6,10 +6,12 @@
 //! RFC 8210. Annoyingly, the format of the `EndOfData` PDU changes between
 //! the two versions.
 
-use std::{io, mem, slice};
+use std::{borrow, error, fmt, io, mem, ops, slice};
 use std::convert::TryFrom;
 use std::marker::Unpin;
-use std::net::{Ipv4Addr, Ipv6Addr};
+use std::net::{IpAddr, Ipv4Addr, Ipv6Addr};
+use bytes::Bytes;
+use routecore::addr::{MaxLenPrefix, Prefix};
 use tokio::io::{
     AsyncRead, AsyncReadExt, AsyncWrite, AsyncWriteExt
 };
@@ -465,44 +467,343 @@ impl Ipv6Prefix {
 concrete!(Ipv6Prefix);
 
 
+//------------ RouterKey -----------------------------------------------------
+
+/// A BGPsec router key.
+pub struct RouterKey {
+    fixed: RouterKeyFixed,
+    key_info: RouterKeyInfo,
+}
+
+#[derive(Default)]
+#[repr(packed)]
+struct RouterKeyFixed {
+    header: Header,
+    key_identifier: [u8; 20],
+    asn: u32,
+}
+
+impl RouterKey {
+    /// The PDU type of a Router Key PDU.
+    pub const PDU: u8 = 9;
+
+    /// The maximum size of the subject public key info data.
+    pub const fn max_key_info_size() -> usize {
+        (u32::MAX as usize) - mem::size_of::<RouterKeyFixed>()
+    }
+
+    /// Creates a new router key PDU.
+    ///
+    /// # Panics
+    ///
+    /// This function panics if the length of the resulting PDU doesn’t fit
+    /// in a `u32`.
+    pub fn new(
+        version: u8,
+        flags: u8,
+        key_identifier: [u8; 20],
+        asn: u32,
+        key_info: RouterKeyInfo,
+    ) -> Self {
+        // We know it fits but let’s be sure.
+        let len = u32::try_from(
+            mem::size_of::<RouterKeyFixed>().checked_add(
+                key_info.len()
+            ).expect("RouterKey RTR PDU size overflow")
+        ).expect("RouterKey RTR PDU size overflow");
+
+        RouterKey {
+            fixed: RouterKeyFixed {
+                header: Header::new(
+                    version, Self::PDU,
+                    (flags as u16) << 8,
+                    len
+                ),
+                key_identifier,
+                asn: asn.to_be(),
+            },
+            key_info
+        }
+    }
+
+    /// Returns the value of the version field of the header.
+    pub fn version(&self) -> u8 {
+        self.fixed.header.version()
+    }
+
+    /// Returns the PDU size.
+    ///
+    /// The size is returned as a `u32` since that type is used in
+    /// the header.
+    pub fn size(&self) -> u32 {
+        (
+            mem::size_of::<RouterKeyFixed>() + self.key_info.len()
+        ) as u32
+    }
+
+    /// Returns the flags field for the router key.
+    ///
+    /// The only flag currently used is the least significant bit that is
+    /// 1 for an announcement and 0 for a withdrawal.
+    pub fn flags(&self) -> u8 {
+        (self.fixed.header.session >> 8) as u8
+    }
+
+    /// Returns the subject key identifier.
+    pub fn key_identifier(&self) -> [u8; 20] {
+        self.fixed.key_identifier
+    }
+
+    /// Returns the ASN.
+    pub fn asn(&self) -> u32 {
+        u32::from_be(self.fixed.asn)
+    }
+
+    /// Returns a reference to the subject key info
+    pub fn key_info(&self) -> &RouterKeyInfo {
+        &self.key_info
+    }
+
+    /// Converts the PDU into the subject key info.
+    pub fn into_key_info(self) -> RouterKeyInfo {
+        self.key_info
+    }
+
+    /// Writes a value to a writer.
+    pub async fn write<A: AsyncWrite + Unpin>(
+        &self,
+        a: &mut A
+    ) -> Result<(), io::Error> {
+        a.write_all(self.fixed.as_ref()).await?;
+        a.write_all(self.key_info.as_ref()).await
+    }
+
+    /// Reads a value from a reader.
+    ///
+    /// If a value with a different PDU type is received, returns an
+    /// error.
+    pub async fn read<Sock: AsyncRead + Unpin>(
+        sock: &mut Sock 
+    ) -> Result<Self, io::Error> {
+        let mut fixed = RouterKeyFixed::default();
+        sock.read_exact(fixed.header.as_mut()).await?;
+        if fixed.header.pdu() != Self::PDU {
+            return Err(io::Error::new(
+                io::ErrorKind::InvalidData,
+                "PDU type mismatch when expecting router key"
+            ))
+        }
+        let info_len = match
+            (fixed.header.length() as usize).checked_sub(fixed.as_ref().len())
+        {
+            Some(len) => len,
+            None => {
+                return Err(io::Error::new(
+                    io::ErrorKind::InvalidData,
+                    "invalid length for router key"
+                ))
+            }
+        };
+        sock.read_exact(&mut fixed.as_mut()[Header::LEN..]).await?;
+        let key_info = RouterKeyInfo::read(sock, info_len).await?;
+        Ok(RouterKey { fixed, key_info })
+    }
+
+    /// Reads only the payload part of a value from a reader.
+    pub async fn read_payload<Sock: AsyncRead + Unpin>(
+        header: Header, sock: &mut Sock
+    ) -> Result<Self, io::Error> {
+        let info_len = match
+            (header.length() as usize).checked_sub(
+                mem::size_of::<RouterKeyFixed>()
+            )
+        {
+            Some(len) => len,
+            None => {
+                return Err(io::Error::new(
+                    io::ErrorKind::InvalidData,
+                    "invalid length for router key"
+                ))
+            }
+        };
+        let mut fixed = RouterKeyFixed::default();
+        sock.read_exact(&mut fixed.as_mut()[Header::LEN..]).await?;
+        let key_info = RouterKeyInfo::read(sock, info_len).await?;
+        Ok(RouterKey { fixed, key_info })
+    }
+}
+
+impl AsRef<[u8]> for RouterKeyFixed {
+    fn as_ref(&self) -> &[u8] {
+        unsafe {
+            slice::from_raw_parts(
+                self as *const Self as *const u8,
+                mem::size_of::<Self>()
+            )
+        }
+    }
+}
+
+impl AsMut<[u8]> for RouterKeyFixed {
+    fn as_mut(&mut self) -> &mut [u8] {
+        unsafe {
+            slice::from_raw_parts_mut(
+                self as *mut Self as *mut u8,
+                mem::size_of::<Self>()
+            )
+        }
+    }
+}
+
+
+//------------ RouterKeyInfo -------------------------------------------------
+
+/// The subject public key info data of a router key.
+///
+/// This is a simple newtype around a `Bytes` enforcing a size limit so that
+/// a value can always be used in a router key PDU.
+#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+pub struct RouterKeyInfo(Bytes);
+
+impl RouterKeyInfo {
+    /// Creates a new value from some bytes.
+    ///
+    /// Returns an error if the bytes are too large to fit into a router key
+    /// PDU.
+    pub fn new(bytes: Bytes) -> Result<Self, KeyInfoError> {
+        if bytes.len() > RouterKey::max_key_info_size() {
+            Err(KeyInfoError)
+        }
+        else {
+            Ok(RouterKeyInfo(bytes))
+        }
+    }
+
+    /// Returns a reference to the octets of the value.
+    pub fn as_slice(&self) -> &[u8] {
+        self.0.as_ref()
+    }
+
+    /// Converts the value into the underlying bytes.
+    pub fn into_bytes(self) -> Bytes {
+        self.0
+    }
+
+    /// Reads a value of the given length from the buffer.
+    async fn read<Sock: AsyncRead + Unpin>(
+        sock: &mut Sock, len: usize,
+    ) -> Result<Self, io::Error> {
+        let mut key_info = vec![0u8; len];
+        sock.read_exact(key_info.as_mut()).await?;
+        Ok(RouterKeyInfo(key_info.into()))
+    }
+}
+
+
+//--- TryFrom
+
+impl TryFrom<Vec<u8>> for RouterKeyInfo {
+    type Error = KeyInfoError;
+
+    fn try_from(src: Vec<u8>) -> Result<Self, Self::Error> {
+        Self::new(src.into())
+    }
+}
+
+impl TryFrom<Bytes> for RouterKeyInfo {
+    type Error = KeyInfoError;
+
+    fn try_from(src: Bytes) -> Result<Self, Self::Error> {
+        Self::new(src)
+    }
+}
+
+
+//--- Deref, AsRef, and Borrow
+
+impl ops::Deref for RouterKeyInfo {
+    type Target = Bytes;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl AsRef<Bytes> for RouterKeyInfo {
+    fn as_ref(&self) -> &Bytes {
+        &self.0
+    }
+}
+
+impl AsRef<[u8]> for RouterKeyInfo {
+    fn as_ref(&self) -> &[u8] {
+        self.0.as_ref()
+    }
+}
+
+impl borrow::Borrow<Bytes> for RouterKeyInfo {
+    fn borrow(&self) -> &Bytes {
+        self.as_ref()
+    }
+}
+
+impl borrow::Borrow<[u8]> for RouterKeyInfo {
+    fn borrow(&self) -> &[u8] {
+        self.as_ref()
+    }
+}
+
+
 //------------ Payload -------------------------------------------------------
 
 /// All possible payload types.
+#[non_exhaustive]
 pub enum Payload {
     /// An IPv4 prefix.
     V4(Ipv4Prefix),
 
     /// An IPv6 prefix.
     V6(Ipv6Prefix),
+
+    /// A router key.
+    RouterKey(RouterKey),
 }
 
 impl Payload {
     /// Creates an payload value for the given payload.
-    pub fn new(version: u8, flags: u8, payload: payload::Payload) -> Self {
+    pub fn new(version: u8, flags: u8, payload: &payload::Payload) -> Self {
         match payload {
-            payload::Payload::V4(prefix) => {
-                Payload::V4(
-                    Ipv4Prefix::new(
-                        version,
-                        flags,
-                        prefix.prefix_len,
-                        prefix.max_len,
-                        prefix.prefix,
-                        prefix.asn
-                    )
-                )
+            payload::Payload::Origin(origin) => {
+                match origin.prefix.addr() {
+                    IpAddr::V4(addr) => {
+                        Payload::V4(Ipv4Prefix::new(
+                            version,
+                            flags,
+                            origin.prefix.prefix_len(),
+                            origin.prefix.resolved_max_len(),
+                            addr,
+                            origin.asn.into()
+                        ))
+                    }
+                    IpAddr::V6(addr) => {
+                        Payload::V6(Ipv6Prefix::new(
+                            version,
+                            flags,
+                            origin.prefix.prefix_len(),
+                            origin.prefix.resolved_max_len(),
+                            addr,
+                            origin.asn.into()
+                        ))
+                    }
+                }
             }
-            payload::Payload::V6(prefix) => {
-                Payload::V6(
-                    Ipv6Prefix::new(
-                        version,
-                        flags,
-                        prefix.prefix_len,
-                        prefix.max_len,
-                        prefix.prefix,
-                        prefix.asn
-                    )
-                )
+            payload::Payload::RouterKey(key) => {
+                Payload::RouterKey(RouterKey::new(
+                    version, flags,
+                    key.key_identifier.into(),
+                    key.asn.into(),
+                    key.key_info.clone()
+                ))
             }
         }
     }
@@ -535,16 +836,10 @@ impl Payload {
                     Ok(Some(Payload::V6(res)))
                 })
             }
-            9 => { // Router Key
-                // Let’s skip over this in blocks of one 1k.
-                let mut buf = [0u8; 1024];
-                let mut len = header.length() as usize;
-                while len > 1024 {
-                    sock.read_exact(&mut buf).await?;
-                    len -= 1024;
-                }
-                sock.read_exact(&mut buf[..len]).await?;
-                Ok(Ok(None))
+            RouterKey::PDU => {
+                RouterKey::read_payload(header, sock).await.map(|res| {
+                    Ok(Some(Payload::RouterKey(res)))
+                })
             }
             EndOfData::PDU => {
                 EndOfData::read_payload(header, sock).await.map(Err)
@@ -563,6 +858,7 @@ impl Payload {
         match *self {
             Payload::V4(ref data) => data.version(),
             Payload::V6(ref data) => data.version(),
+            Payload::RouterKey(ref key) => key.version(),
         }
     }
 
@@ -571,6 +867,7 @@ impl Payload {
         match *self {
             Payload::V4(ref data) => data.flags(),
             Payload::V6(ref data) => data.flags(),
+            Payload::RouterKey(ref key) => key.flags(),
         }
     }
 
@@ -579,49 +876,70 @@ impl Payload {
         &self,
         a: &mut A
     ) -> Result<(), io::Error> {
-        a.write_all(self.as_ref()).await
+        match *self {
+            Payload::V4(ref data) => data.write(a).await,
+            Payload::V6(ref data) => data.write(a).await,
+            Payload::RouterKey(ref data) => data.write(a).await,
+        }
     }
 
     /// Converts the payload PDU into action and payload.
-    pub fn to_payload(&self) -> (payload::Action, payload::Payload) {
-        (
-            payload::Action::from_flags(self.flags()),
-            match self {
+    ///
+    /// Returns an error if the PDU isn’t acceptable for some reason.
+    pub fn to_payload(
+        &self
+    ) -> Result<(payload::Action, payload::Payload), Error> {
+        fn make_payload(
+            payload: &Payload,
+        ) -> Result<payload::Payload, &'static str> {
+            match payload {
                 Payload::V4(data) => {
-                    payload::Payload::V4(payload::Ipv4Prefix {
-                        prefix: data.prefix(),
-                        prefix_len: data.prefix_len(),
-                        max_len: data.max_len(),
-                        asn: data.asn(),
-                    })
+                    Ok(payload::Payload::origin(
+                        MaxLenPrefix::new(
+                            Prefix::new_v4_relaxed(
+                                data.prefix(), data.prefix_len()
+                            )?,
+                            Some(data.max_len())
+                        )?,
+                        data.asn().into()
+                    ))
                 }
                 Payload::V6(data) => {
-                    payload::Payload::V6(payload::Ipv6Prefix {
-                        prefix: data.prefix(),
-                        prefix_len: data.prefix_len(),
-                        max_len: data.max_len(),
-                        asn: data.asn(),
-                    })
+                    Ok(payload::Payload::origin(
+                        MaxLenPrefix::new(
+                            Prefix::new_v6_relaxed(
+                                data.prefix(), data.prefix_len()
+                            )?,
+                            Some(data.max_len())
+                        )?,
+                        data.asn().into()
+                    ))
+                }
+                Payload::RouterKey(key) => {
+                    Ok(payload::Payload::router_key(
+                        key.key_identifier().into(), key.asn().into(),
+                        key.key_info().clone()
+                    ))
                 }
             }
-        )
-    }
-}
+        }
 
-impl AsRef<[u8]> for Payload {
-    fn as_ref(&self) -> &[u8] {
+        Ok((
+            payload::Action::from_flags(self.flags()),
+            make_payload(self).map_err(|text| {
+                Error::new(
+                    self.version(), 0, self.as_partial_slice(), text.as_bytes()
+                )
+            })?
+        ))
+    }
+
+    /// Returns an octets slice of as much of the PDU as possible.
+    pub fn as_partial_slice(&self) -> &[u8] {
         match *self {
             Payload::V4(ref prefix) => prefix.as_ref(),
             Payload::V6(ref prefix) => prefix.as_ref(),
-        }
-    }
-}
-
-impl AsMut<[u8]> for Payload {
-    fn as_mut(&mut self) -> &mut [u8] {
-        match *self {
-            Payload::V4(ref mut prefix) => prefix.as_mut(),
-            Payload::V6(ref mut prefix) => prefix.as_mut(),
+            Payload::RouterKey(ref key) => key.fixed.as_ref(),
         }
     }
 }
@@ -855,7 +1173,7 @@ pub struct CacheReset {
 
 impl CacheReset {
     /// The PDU type for a cache reset.
-    pub const PDU: u8 = 7;
+    pub const PDU: u8 = 8;
 
     /// Creates a cache reset.
     pub fn new(version: u8) -> Self {
@@ -1029,3 +1347,18 @@ impl Header {
 
 common!(Header);
 
+
+//============ ErrorTypes ====================================================
+
+/// The key info of a router key was too large.
+#[derive(Clone, Copy, Debug)]
+pub struct KeyInfoError;
+
+impl fmt::Display for KeyInfoError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.write_str("router key size overflow")
+    }
+}
+
+impl error::Error for KeyInfoError { }
+
